#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : __init__.py
# @Time         : 2023/8/4 10:33
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://github.com/mher/flower/tree/master
"""
flower是基于web的监控和管理Celery的工具，和任务队列是隔离的，flower的运行并不会影响到任务队列的真正执行。
flower作为celery后台任务的管理工具，将各个任务的执行情况、各个worker的健康状态进行实时监控并以可视化的方式展现。
"""
from celery import Celery, shared_task, states

from meutils.pipe import *
from meutils.serving.celery import config


Celery().conf.from_object(config)



